#!/bin/tcsh

### This script analyzes one subject's timeseries data (block design)
### It must be run after Script_1.warper.csh

# argv[1] = stim filename       [e.g., blk10stim.Beijing.txt]
# argv[2] = stim response       [e.g., 'BLOCK(10,1)']
# argv[3] = blur radius         [e.g., 4, 6, 8, 10]
# argv[4] = subject ID to run   [e.g., sub00156]

if( $#argv < 4 )then
  echo "Need at least 4 args"
  exit 1
endif

# set some variables if we are running SLURM

if( $?SLURM_CPUS_PER_TASK )then
  setenv OMP_NUM_THREADS $SLURM_CPUS_PER_TASK
endif

if( $?SLURM_JOBID )then
  set tempdir = /lscratch/\$SLURM_JOBID
else
  set tempdir = .
endif

# don't log AFNI programs in ~/.afni.log
# don't try any version checks
# don't auto-compress output files

setenv AFNI_DONT_LOGFILE  YES
setenv AFNI_VERSION_CHECK NO
setenv AFNI_COMPRESSOR    NONE

# set the base dataset for MNI-izing
# (not actually needed since this was done in Script_1.warper.csh)

set basedset = MNI152_2009_template.nii.gz
set tpath = `@FindAfniDsetPath $basedset`
if( "$tpath" == '' ) then
  echo "***** @SSwarper -- Failed to find template $basedset -- exiting :("
  exit 1
endif
set basedset = $tpath/$basedset

### set the data directories we will use

# topdir = directory above this Scripts directory

set topdir  = `dirname $cwd`

set restdir = $topdir/rest_orig
set anatdir = $topdir/anat_orig
set warpdir = $topdir/anat_warped
set stimdir = $topdir/stimfiles

set start_directory = $cwd

# extract arg 1 and check it for existence

set stimfile = $argv[1]
if( ! -f $stimfile ) set stimfile = $stimdir/$stimfile
if( ! -f $stimfile ) then
  echo "argv[1] = $stimfile DOES NOT EXIST"
  exit 1
endif

# set the 'name' (label) for this stimulus in the output files

set stimfname = `basename $stimfile`
set stimname  = `basename $stimfile .txt`
set stimname  = `basename $stimname .1D`
set stimname  = `basename $stimname .Beijing`
set stimname  = `basename $stimname .Cambridge`

# extract arg 2 (HRF model)

set stimresp = $argv[2]

# extract arg 3 (blurring in mm)

set blur =     $argv[3]
set blurname = b${blur}mm

# set subject ID from arg 4

set subj = $argv[4]

# create output directory, if needed

set outdir = $topdir/$stimname.$blurname
mkdir -p $outdir
echo
echo "===== Output directory is $outdir"
echo

# copy stimfile there

if( ! -f $outdir/$stimfname )then
  cp -f $stimfile $outdir/$stimfname
endif

cd $outdir
mkdir -p snapshots

# Process this one subject

set rest_dset = $restdir/rest_${subj}.nii.gz
set anat_dset = $warpdir/anatSS.${subj}.nii.gz

# if we ran it before but it failed for some reason,
# kill the old version of the results and try again

  if( -d ${subj}.results )then
    if( ! -f ${subj}.results/stats.${subj}_REML+tlrc.HEAD )then
      \rm -rf *${subj}*
    endif
  endif

# Run this if the results don't exist and the data does

  if( ! -d ${subj}.results && -f $anat_dset && -f $rest_dset )then

    echo "-------------------------------"
    echo "Processing $subj"
    echo "-------------------------------"

# run afni_proc.py to create a single subject processing script

    afni_proc.py -subj_id $subj                                   \
         -script proc.$subj    -scr_overwrite                     \
         -blocks despike tshift align tlrc volreg                 \
                 blur mask scale regress                          \
         -copy_anat $anat_dset                                    \
            -anat_has_skull no                                    \
         -dsets $rest_dset                                        \
         -tcat_remove_first_trs 0                                 \
         -align_opts_aea -giant_move                              \
             -cost lpc+ZZ                                         \
         -volreg_align_to MIN_OUTLIER                             \
         -volreg_align_e2a                                        \
         -volreg_tlrc_warp                                        \
         -tlrc_base $basedset                                     \
         -tlrc_NL_warp                                            \
         -tlrc_NL_warped_dsets                                    \
               $warpdir/anatQQ.${subj}.nii.gz                     \
               $warpdir/anatQQ.${subj}.aff12.1D                   \
               $warpdir/anatQQ.${subj}_WARP.nii.gz                \
         -volreg_warp_dxyz 3                                      \
         -blur_size $blur                                         \
         -regress_anaticor_fast                                   \
         -regress_anaticor_fwhm 20                                \
         -regress_stim_times $stimfile                            \
         -regress_stim_labels $stimname                           \
         -regress_basis "$stimresp"                               \
         -regress_censor_motion 0.2                               \
         -regress_censor_outliers 0.02                            \
         -regress_3dD_stop                                        \
         -regress_make_ideal_sum sum_ideal.1D                     \
         -regress_est_blur_errts                                  \
         -regress_reml_exec                                       \
         -regress_run_clustsim no

     # Run analysis
     tcsh -xef proc.${subj} >& proc.${subj}.output

     # If it worked, run the volreg snapshots and compress outputs
     if( -d ${subj}.results )then
       cd ${subj}.results
       @snapshot_volreg anat_final.${subj}+tlrc.HEAD pb0?.${subj}.r01.volreg+tlrc.HEAD ${subj}
       if( -f ${subj}.jpg ) \mv -f ${subj}.jpg ../snapshots/
       gzip -1v *.BRIK *.nii
       cd ..
     endif
  else
    echo "----------------------------------------------------------------------------"
    echo "Skipping $subj"
    if( -d ${subj}.results ) echo "  ${subj}.results EXISTS"
    if( ! -f $anat_dset    ) echo "  $anat_dset DOES NOT EXIST"
    if( ! -f $rest_dset    ) echo "  $rest_dset DOES NOT EXIST"
    echo "----------------------------------------------------------------------------"
  endif

echo
echo "===== Finished ====="
echo

time
exit 0
