#!/bin/tcsh

##### This file does several different tests:
#####   over a range of per-voxel p-value thresholds
#####   each one with 1-sided and 2-sided t-tests
##### With ETAC (3dXClustsim), ClustSim on residuals,
##### and ClustSim with 4 different parametric models:
#####  * ACF from t-test resids
#####  * FWHM from t-test resids
#####  * mean ACF from subjects
#####  * mean FWHM from subjects
##### The FWHM estimates from the mixed ACF fit, but then
##### those 3dClustSim results are from the Gaussian model.

##### This particular script is for 2-sided tests,
##### which is controlled by setting Anum and Bnum, far below.

##### set some settings for the setup

unset noclobber
set echo

# set thread count?
if( $?SLURM_CPUS_PER_TASK )then
  setenv OMP_NUM_THREADS $SLURM_CPUS_PER_TASK
else
  unsetenv OMP_NUM_THREADS
endif

# set temporary workspace dir?
if( $?SLURM_JOBID )then
  set tempdir = /lscratch/\$SLURM_JOBID
else
  set tempdir = .
endif

setenv AFNI_COMPRESSOR   NONE
setenv AFNI_NOPLUGINS    YES
setenv AFNI_DONT_LOGFILE YES

##### set variables from the arguments
#      stim = stimulus name
#      blur = how many mm?
#      case = name for these output files

set stim = $argv[1]
set blur = $argv[2]
set case = $argv[3]

# seed = random number seed for generating subject lists
set seed = `count -dig 1 1 999999999 R1`

# used once later
set site = Beijing

# directory with all the good stuff (directory above this one)
set topdir = `dirname $cwd`

# directory with the single subject analyzed data for this stim+blur
set wdir = $topdir/${stim}stim.b${blur}mm

# directory with all the scripts for this site (this one)
set scdir = $topdir/Scripts

##### set various parameters for the clustering stuff

# NN clustering (NN1 or NN2 or NN3)
set nnlev = NN2

# pval table generated by 3dClustSim -LOTS
set pthr  = (  0.10     0.09     0.08     0.07     0.06              \
               0.05     0.04     0.03     0.02     0.015     0.01    \
               0.007    0.005    0.003    0.002    0.0015    0.001   \
               0.0007   0.0005   0.0003   0.0002   0.00015   0.0001  \
               0.00007  0.00005  0.00003  0.00002  0.000015  0.00001  )

# select which pvals to use by choosing row number in pthr table
#   [e.g., prow=15 --> pval=0.002]
# then select the actual pvals from the pthr table
# then set index for 1dcat-ing correct cluster threshold from 3dClustSim output
#   1D file (must reduce prow by 1 since 1dcat uses 0-based row indexes)
# then set z-stat threshold arrays (1 and 2 sided tests)

set prow_list = ( 11   13   15   17   )
# names used for each prow
set Xnam_list = ( P010 P005 P002 P001 )
# these lists will be filled out in the loop below:
#  pval_list = actual p-values
set pval_list = ( $prow_list )
#  Cind_list = list of indexes into 3dClustSim output table
set Cind_list = ( $prow_list )
#  zth1_list = list of 1-sided thresholds for t (z) tests
set zth1_list = ( $prow_list )
#  zth1_list = list of 2-sided thresholds for t (z) tests
set zth2_list = ( $prow_list )
# number of p-values we are testing
set nump      = $#prow_list
# twice that (because we do 1- and 2-sided)
@   nump2     = 2 * $nump

foreach pind ( `count -dig 1 1 $nump` )
  set prow             = $prow_list[$pind]
  set pval             = $pthr[$prow]
  set pval_list[$pind] = $pval
  @ ppp                = $prow - 1
  set Cind_list[$pind] = "{$ppp}[6]"
  set zth1_list[$pind] = `ccalc "qginv($pval)"`
  set zth2_list[$pind] = `ccalc "qginv($pval/2)"`
end

# compute rmm for 3dmerge from NN type
if( $nnlev == "NN1" )then
  set rmm = 1.111
else if( $nnlev == "NN2" )then
  set rmm = 1.555
else
  set rmm = 1.888
endif

##### change to data directory, get ready to work

cd $wdir

# setup the command to run 3dttest++ -Xclustsim

# number of subjects in each set (Bnum=0 --> 1-sample t-test)
set Anum = 20
set Bnum = 20

# sdir = subject data top dir
set sdir    = .
# odir = output dir for this simulation
set odir    = TT.$Anum.$Bnum
# prefix for Xclustsim results
set prefix  = X.$case
set Xprefix = $prefix
set Xttset  = ${prefix}.nii.gz

# make the output directory (if it doesn't exist yet)
mkdir -p $odir
# remove any earlier instances of files for this case
find . $odir -maxdepth 1 -name \*.$case.\* -exec \rm -rf {} \;

# get list of all subjects (for random selection in make_one_ttest_command.csh)
set Slist   = ( `cat $scdir/$site.list.txt` )

# set mask dataset
set mask    = $topdir/basemask.nii.gz

# extra stuff for the 3dttest++ command generation
set Olist   = ( -Xclustsim -prefix_clustsim $prefix )

# loop through the cases to simulate, adding commands to 3dttest++
# for each 3dXClustSim case, and making the lists of 3dXClustSim
# output files

set Xmthresh_list = ( )
set Xhitmap_list  = ( )

foreach pind ( `count -dig 1 1 $nump` )
# 1-sided 3dXClustSim
  set Olist  = ( $Olist -Xclu_opt name=${Xnam_list[$pind]}s1:${nnlev}:1sid:pthr=${pval_list[$pind]} )
  set Xmthresh_list = ( $Xmthresh_list ${prefix}.${Xnam_list[$pind]}s1.CsimX.mthresh.nii )
  set Xhitmap_list  = ( $Xhitmap_list  ${prefix}.${Xnam_list[$pind]}s1.hitmap.nii.gz )

# 2-sided 3dXClustSim
  set Olist = ( $Olist -Xclu_opt name=${Xnam_list[$pind]}s2:${nnlev}:2sid:pthr=${pval_list[$pind]} )
  set Xmthresh_list = ( $Xmthresh_list ${prefix}.${Xnam_list[$pind]}s2.CsimX.mthresh.nii )
  set Xhitmap_list  = ( $Xhitmap_list  ${prefix}.${Xnam_list[$pind]}s2.hitmap.nii.gz )
end

# actually create the first 3dttest++ command
set noglob
source $scdir/make_one_tt_command.csh
unset noglob

# and put the command into a file to be run below
echo "$ttcmd" > $odir/tt.$case.csh

##### now setup the command for the 3dttest++ -Clustsim run

# prefix for these results

set prefix  = C.$case
set Cprefix = $prefix
set Olist   = ( -Clustsim -prefix_clustsim $prefix -ACF )

set Cttest         = ${prefix}.nii.gz
set CACFresid      = ${prefix}.resid.ACFparam.txt

# suffix for various 3dClustSim output files to be produced
set C1Dfiles1_suff = CSim.${nnlev}_1sided.1D
set C1Dfiles2_suff = CSim.${nnlev}_2sided.1D
set C1Dfiles1      = ${prefix}.${C1Dfiles1_suff}
set C1Dfiles2      = ${prefix}.${C1Dfiles2_suff}

# loop over the cases to run and make the list of the hitmap
# files that will be produced later

set Chitmaps1_suff = ( )
set Chitmaps2_suff = ( )
foreach pind ( `count -dig 1 1 $nump` )
   set Chitmaps1_suff = ( $Chitmaps1_suff ${Xnam_list[$pind]}s1.hitmap.nii.gz )
   set Chitmaps2_suff = ( $Chitmaps2_suff ${Xnam_list[$pind]}s2.hitmap.nii.gz )
end

# actually create the second 3dttest++ command
set noglob
source $scdir/make_one_tt_command.csh
unset noglob

# and put the command into the file
echo "$ttcmd" >> $odir/tt.$case.csh

##### now run the t-test commands (this is where most of the CPU time is)

tcsh $odir/tt.$case.csh

##### now analyze the analyses

cd $odir

# if t-stat output files don't exist, our life is ruined :(

if( ! -f $Xttset )then
  echo "ERROR: failed to create $Xttset -- exiting" ; exit 1
endif

if( ! -f $Cttest )then
  echo "ERROR: failed to create $Cttest -- exiting" ; exit 1
endif

##### multi-threshold from 3dXClustSim
##### -nozero option means no hitmap is output if nothing survives

foreach ppp ( `count -dig 1 1 $nump2` )
  set Xmtset = $Xmthresh_list[$ppp]
  set Xhmset = $Xhitmap_list[$ppp]
  if( ! -f ../$Xmtset )then
    echo "ERROR: can't find $Xmtset -- skipping" ; continue
  endif
  mv ../$Xmtset .
  3dMultiThresh -quiet -prefix $Xhmset -nozero -mthresh $Xmtset -input $Xttset
  gzip -1v $Xmtset
end

##### now deal with the Clustsim run

# mv and rm various other files from the Clustsim run
\mv -vf ../${C1Dfiles1} ../${C1Dfiles2} ../$CACFresid .
\rm -vf ../${Cprefix}.CSim.* ../${Cprefix}.resid.ACF*

# extract the ACF parameters from the various subjects, and compute their mean
# (TTlist was set in make_one_tt_command earlier)

# create a unique temp file name
set tfil = `3dnewid -fun11`
set tfil = junkACF.$tfil.1D
if( -f $tfil ) \rm -f $tfil
touch $tfil
# and get the row of ACF parameters from each subject
foreach sub ( $TTlist )
  1dcat "../${sub}.results/blur_est.${sub}.1D{1}"  >> $tfil
end
# and compute their mean (and save into output file)
set ACFsubj = ( `1dsum -mean $tfil` )
echo $ACFsubj > $prefix.ACFsubj.1D
\rm -vf $tfil

# get ACF parameters from the t-test residuals
set ACFresid = ( `cat $CACFresid` )

echo "ACFsubj  = $ACFsubj"
echo "ACFresid = $ACFresid"

# run 3dClustSim 4 different ways (in a temp directory)

set jdir = `3dnewid -fun11`
set jdir = junk.${Cprefix}.${jdir}
mkdir -p $jdir
cd $jdir

# D = ACF from resids
3dClustSim -mask $mask -acf $ACFresid[1] $ACFresid[2] $ACFresid[3] \
           -LOTS -nodec -prefix D.${case}.CSim
set D1Dfiles1 = D.${case}.CSim.${nnlev}_1sided.1D
set D1Dfiles2 = D.${case}.CSim.${nnlev}_2sided.1D
\mv -vf $D1Dfiles1 $D1Dfiles2 ..

# E = FWHM from resids -- must scale down by sqrt(2)
set FWHM = `ccalc $ACFresid[4]/1.41421`
3dClustSim -mask $mask -fwhm $FWHM        \
           -LOTS -nodec -prefix E.${case}.CSim
set E1Dfiles1 = E.${case}.CSim.${nnlev}_1sided.1D
set E1Dfiles2 = E.${case}.CSim.${nnlev}_2sided.1D
\mv -vf $E1Dfiles1 $E1Dfiles2 ..

# F = ACF from subjects
3dClustSim -mask $mask -acf $ACFsubj[1] $ACFsubj[2] $ACFsubj[3] \
           -LOTS -nodec -prefix F.${case}.CSim
set F1Dfiles1 = F.${case}.CSim.${nnlev}_1sided.1D
set F1Dfiles2 = F.${case}.CSim.${nnlev}_2sided.1D
\mv -vf $F1Dfiles1 $F1Dfiles2 ..

# G = FWHM from subjects
set FWHM = `ccalc $ACFsubj[4]/1.41421`
3dClustSim -mask $mask -fwhm $FWHM       \
           -LOTS -nodec -prefix G.${case}.CSim
set G1Dfiles1 = G.${case}.CSim.${nnlev}_1sided.1D
set G1Dfiles2 = G.${case}.CSim.${nnlev}_2sided.1D
\mv -vf $G1Dfiles1 $G1Dfiles2 ..

cd ..
\rm -rf $jdir

# now appy the global cluster-size threshold from 3dClustSim runs

# prefix for the output files for 5 cases
set pnam   = (  C          D          E          F          G         )
# 1-sided 3dClustSim tables to use
set pfils1 = ( $C1Dfiles1 $D1Dfiles1 $E1Dfiles1 $F1Dfiles1 $G1Dfiles1 )
# 2-sided 3dClustSim tables to use
set pfils2 = ( $C1Dfiles2 $D1Dfiles2 $E1Dfiles2 $F1Dfiles2 $G1Dfiles2 )
# lists into which we'll save the cluster-size thresholds used
set psiz1  = ( )
set psiz2  = ( )

# loop over the 5 3dClustSim sources and do 1-side and 2-sided tests
# for each p-value threshold

# loop over the 5 cases
foreach jjj ( `count -dig 1 1 $#pnam` )
  set cpref = $pnam[$jjj].$case
  set psiz1 = ( $psiz1 " -- $cpref ::" )
  set psiz2 = ( $psiz2 " -- $cpref ::" )
# loop over the p-values
  foreach ppp ( `count -dig 1 1 $nump` )
# get 1-sided cluster-size threshold
    set cfile = $pfils1[$jjj]
    set csiz  = `1dcat $cfile"$Cind_list[$ppp]"`
    set psiz1 = ( $psiz1 "${Xnam_list[$ppp]} C=$csiz" )
# and apply it
    3dmerge -nozero -1tindex 1 -2thresh -9999 $zth1_list[$ppp] -dxyz=1 \
            -1clust $rmm $csiz -prefix ${cpref}.${Chitmaps1_suff[$ppp]} $Cttest

# get 2-sided cluster-size threshold
    set cfile = $pfils2[$jjj]
    set csiz  = `1dcat $cfile"$Cind_list[$ppp]"`
    set psiz2 = ( $psiz2 "${Xnam_list[$ppp]} C=$csiz" )
# and apply it
    3dmerge -nozero -1tindex 1 -2thresh -$zth2_list[$ppp] $zth2_list[$ppp] -dxyz=1 \
            -1clust $rmm $csiz -prefix ${cpref}.${Chitmaps2_suff[$ppp]} $Cttest
  end
end

# echo the saved cluster-size thresolds for each case to output files
echo "Cluster sizes 1sided: $psiz1" > ${Cprefix}.csize1.txt
echo "Cluster sizes 2sided: $psiz2" > ${Cprefix}.csize2.txt

# vamoose the ranch
time
exit 0
